#
# Author: 
# Description: Derived class for squares on a 2D game board. A CellDoor
#    doesn't need to know exactly where it is on the board, but it does
#    need to know if it is open or locked, and if open, if it contains an item.
#

from CellPassage import CellPassage
from Item import Item
import Hero
import Config
import picture
import StdDraw

class CellDoor(CellPassage):
    
    # Create a new cell in the dungeon with the specified image.
    # The CellDoor class represents an door that can be closed (locked) or open.
    # NOTE: an open door is allowed to hold an item (e.g. a gem or key).
    def __init__(self, imageOpen, imageClosed, locked):
        super().__init__(imageOpen)
        self.imageClosed = imageClosed
        self.locked = locked
        self.closedPic = picture.Picture(imageClosed)

    # Attempt to move the hero to this door location.
    # This will always work if the door is already open.
    # If it is still locked, the hero has to use a key to open door.
    # Returns true if hero made it through, false otherwise
    def attemptToMove(self, hero):
        if self.locked:
            if hero.attemptToUse(Item.KEY):
                self.locked = False
        if self.locked:
            return False
        else:
            return super().attemptToMove(hero)
        
    # Draw at the specified position in the dungeon.
    # Also draws any items contained at this location.
    # NOTE: x and y are indexes in the dungeon, NOT the StdDraw drawing location.
    def draw(self, config, x, y):
        if self.locked:
            StdDraw.picture(self.closedPic, (x + 0.5) * config.cellPixels(), (y + 0.5) * config.cellPixels())
        else:
            super().draw(config, x, y)
    
    # Attempts to add the specified item to this cell.
    # A CellDoor can store at most one item, but only if it is unlocked and open.
    # Returns true if item was added, false otherwise.
    def attemptToAdd(self, item):
        if self.locked:
            return False
        else:
            return super().attemptToAdd(item)
    
    # Open the door, this is needed for testing of CellDoor before Hero is implemented. 
    def open(self):
        self.locked = False

#
# Test program for CellDoor.
#    You need not change this code at this time,
#    but feel free to change for your own testing needs.
#
# To run this code and test your class, type "python CellDoor.py" at the
#    command prompt. Your output should be exactly like what is shown
#    on the assignment page.
#

if __name__ == "__main__":
    # The config object stores defaults for sizes, sounds, and images
    config = Config.Config()
        
    # Fake a dungeon that is 3 x 3
    WIDTH = 3
    HEIGHT = 3
    StdDraw.setCanvasSize(WIDTH * config.cellPixels(), HEIGHT * config.cellPixels())
    StdDraw.setXscale(0, WIDTH * config.cellPixels())
    StdDraw.setYscale(0, HEIGHT * config.cellPixels())
        
    # Create a door that is initially locked
    door = CellDoor("door_open.png", "door_closed.png", True)
    door.draw(config, 0, 0)
    print("Adding gem to closed door, result = %5s" %(str(door.attemptToAdd(Item.GEM))))
    print("Adding key to closed door, result = %5s" %(str(door.attemptToAdd(Item.KEY))))
    door.open()
    door.draw(config, 0, 1)
    print("Adding gem to open door,   result = %5s" %(str(door.attemptToAdd(Item.GEM))))
    print("Adding key to open door,   result = %5s" %(str(door.attemptToAdd(Item.KEY))))
    door.draw(config, 0, 2)
        
    # Create a door that is initially unlocked
    door2 = CellDoor("door_open.png", "door_closed.png", False)
    door2.draw(config, 1, 0)
    door2.open()
    door2.draw(config, 1, 1)
    print("Adding key to open door2,  result = %5s" %(str(door2.attemptToAdd(Item.KEY))))
    print("Adding gem to open door2,  result = %5s\n" %(str(door2.attemptToAdd(Item.GEM))))
    door2.draw(config, 1, 2)

    StdDraw.show(2000)
