#
# Author: 
#
# Hero class that contains the methods needed to create a character in the
#    Dungeon game.
# 

from Item import Item
from Config import Config
import StdDraw
import picture
import StdAudio

class Hero:
    
    # Create a new Hero object at the specified location using the default image.
    # NOTE: x and y are indexes in the dungeon, NOT the StdDraw drawing location.  
    def __init__(self, config, x=0, y=0):
        self.config = config
        self.x      = x
        self.y      = y
        self.image  = config.heroImage()
        self.pic    = picture.Picture(self.image)
        # Sets up a list of items the hero has, with a count in each position
        # Initially the hero has no items
        self.itemCount = [0] * Item.count()

    # Change the image used to display this hero. 
    def setImage(self, image):
        self.image = image
        self.pic = picture.Picture(self.image)
    
    # Draw the Hero at its current location.
    # Be sure to convert the hero's x and y indexes to a StdDraw drawing location. 
    def draw(self):
        StdDraw.picture(self.pic, (self.x + 0.5) * self.config.cellPixels(), (self.y + 0.5) * self.config.cellPixels())
    
    # Attempts to use the specified item.
    # This consumes one of the specified items (if the hero has one).
    # If successful, also plays sound effect for using the item.
    # If a given item has no sound effect, don't play anything (but don't crash).
    # Returns true if item was available, false otherwise.
    def attemptToUse(self, item):
        if self.itemCount[item.value] > 0:
            effect = self.config.mapItemToUseSound(item)
            if len(effect) > 0:
                StdAudio.playFile(effect)
            ##### YOUR CODE HERE #####
            return True
        else:
            return False
           
    # Returns how many of the specified item the hero currently has. 
    def getNumItems(self, item):
        return self.itemCount[item.value]
        
    # Called when the hero takes an item from the dungeon.
    # Also plays sound effect for collecting this item.
    # If a given item has no sound effect, don't play anything (but don't crash).
    def collectItem(self, item):
        self.itemCount[item.value] += 1
        effect = self.config.mapItemToCollectSound(item)
        if len(effect) > 0:
            StdAudio.playFile(effect)
    
    # Returns the current x-index of the hero in the dungeon. 
    def getX(self):
        return self.x
    
    # Returns the current y-index of the hero in the dungeon.
    def getY(self):
        return self.y
    
    # Move the hero to the specified x-index and y-index
    def setLocation(self, x, y):
        self.x = x
        self.y = y
    
#
# Test program for Hero.
#    You need not change this code at this time,
#    but feel free to change for your own testing needs.
#
# To run this code and test your class, type "python Hero.py" at the
#    command prompt. Your output should be exactly like what is shown
#    on the assignment page.
#

if __name__ == "__main__":
    # The config object stores defaults for sizes, sounds, and images
    config = Config()
        
    # Fake a dungeon that is 3 x 3
    WIDTH = 3
    HEIGHT = 3
    StdDraw.setCanvasSize(WIDTH * config.cellPixels(), HEIGHT * config.cellPixels())
    StdDraw.setXscale(0, WIDTH * config.cellPixels())
    StdDraw.setYscale(0, HEIGHT * config.cellPixels())

    # Create a default girl hero at (0, 0)
    hero1 = Hero(config, 0, 0)
    hero1.draw()
    print("hero1, location (%d, %d)" %(hero1.getX(), hero1.getY()))
        
    # Test out adding gems and keys
    print("hero1, gems = %d, keys = %d" %(hero1.getNumItems(Item.GEM), hero1.getNumItems(Item.KEY)))
    hero1.collectItem(Item.GEM)
    print("hero1, gems = %d, keys = %d" %(hero1.getNumItems(Item.GEM), hero1.getNumItems(Item.KEY)))
    hero1.collectItem(Item.KEY)
    print("hero1, gems = %d, keys = %d" %(hero1.getNumItems(Item.GEM), hero1.getNumItems(Item.KEY)))
    hero1.collectItem(Item.KEY)
    print("hero1, gems = %d, keys = %d" %(hero1.getNumItems(Item.GEM), hero1.getNumItems(Item.KEY)))
    hero1.collectItem(Item.GEM)
    print("hero1, gems = %d, keys = %d" %(hero1.getNumItems(Item.GEM), hero1.getNumItems(Item.KEY)))
        
    # Now use some items, pausing between so we can hear any sound effects
    for i in range(0,3):
        print("hero1, using key = %5s, gems = %d, keys = %d" %(str(hero1.attemptToUse(Item.KEY)), hero1.getNumItems(Item.GEM), hero1.getNumItems(Item.KEY)))
        StdDraw.show(1000)
    for i in range(0,3):
        print("hero1, using gem = %5s, gems = %d, keys = %d" %(str(hero1.attemptToUse(Item.GEM)), hero1.getNumItems(Item.GEM), hero1.getNumItems(Item.KEY)))
        StdDraw.show(1000)
                
    # Change to the girl into a boy and move up one
    hero1.setImage("character-dragon.png")
    hero1.setLocation(0, 1)
    hero1.draw()
    StdDraw.show(1000)
        
    # Create a boy hero at (1, 0)
    hero2 = Hero(config, 1, 0)
    hero2.setImage("character-dwarf.png")
    hero2.draw()
    StdDraw.show(1000)
    print("hero2, location (%d, %d)" %(hero2.getX(), hero2.getY()))
